@@ -1,8 +1,8 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2016 Andrew Ziem
-# http://www.bleachbit.org
+# Copyright (C) 2008-2019 Andrew Ziem
+# https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,36 +22,35 @@
 Perform (or assist with) cleaning operations.
 """
 
+from __future__ import absolute_import, print_function
+
+from bleachbit import _, expanduser, expandvars
+from bleachbit.FileUtilities import children_in_directory
+from bleachbit.Options import options
+from bleachbit import Command, FileUtilities, Memory, Special
 
 import glob
 import logging
 import os.path
 import re
 import sys
-import traceback
 import warnings
 
-import Command
-import FileUtilities
-import Memory
-import Special
-
 if 'posix' == os.name:
-    import Unix
+    from bleachbit import Unix
 elif 'nt' == os.name:
-    import Windows
-
-from Common import _
-from FileUtilities import children_in_directory, expandvars
-from Options import options
+    from bleachbit import Windows
 
 # Suppress GTK warning messages while running in CLI #34
 warnings.simplefilter("ignore", Warning)
 
 try:
     import gtk
-    HAVE_GTK = True
-except:
+    # Import was successful, but is a display available?
+    HAVE_GTK = gtk.gdk.get_default_root_window() is not None
+except (ImportError, RuntimeError) as e:
+    # ImportError happens when GTK is not installed.
+    # RuntimeError can happen when X is not available (e.g., cron, ssh).
     HAVE_GTK = False
 
 # a module-level variable for holding cleaners
@@ -70,6 +69,7 @@ class Cleaner:
         self.options = {}
         self.running = []
         self.warnings = {}
+        self.regexes_compiled = []
 
     def add_action(self, option_id, action):
         """Register 'action' (instance of class Action) to be executed
@@ -96,9 +96,10 @@ class Cleaner:
                 for ds in self.get_deep_scan(option_id):
                     if isinstance(ds, dict):
                         return False
-            except:
-                print 'warning: exception in auto_hide(), cleaner=%s, option=%s' % (self.name, option_id)
-                traceback.print_exc()
+            except Exception:
+                logger = logging.getLogger(__name__)
+                logger.exception('exception in auto_hide(), cleaner=%s, option=%s',
+                                 self.name, option_id)
         return True
 
     def get_commands(self, option_id):
@@ -107,7 +108,7 @@ class Cleaner:
             if option_id == action[0]:
                 for cmd in action[1].get_commands():
                     yield cmd
-        if not self.options.has_key(option_id):
+        if option_id not in self.options:
             raise RuntimeError("Unknown option '%s'" % option_id)
 
     def get_deep_scan(self, option_id):
@@ -116,7 +117,7 @@ class Cleaner:
             if option_id == action[0]:
                 for ds in action[1].get_deep_scan():
                     yield ds
-        if not self.options.has_key(option_id):
+        if option_id not in self.options:
             raise RuntimeError("Unknown option '%s'" % option_id)
 
     def get_description(self):
@@ -152,22 +153,24 @@ class Cleaner:
 
     def is_running(self):
         """Return whether the program is currently running"""
+        logger = logging.getLogger(__name__)
         for running in self.running:
             test = running[0]
             pathname = running[1]
             if 'exe' == test and 'posix' == os.name:
                 if Unix.is_running(pathname):
-                    print "debug: process '%s' is running" % pathname
+                    logger.debug("process '%s' is running", pathname)
                     return True
             elif 'exe' == test and 'nt' == os.name:
                 if Windows.is_process_running(pathname):
-                    print "debug: process '%s' is running" % pathname
+                    logger.debug("process '%s' is running", pathname)
                     return True
             elif 'pathname' == test:
-                expanded = os.path.expanduser(expandvars(pathname))
+                expanded = expanduser(expandvars(pathname))
                 for globbed in glob.iglob(expanded):
                     if os.path.exists(globbed):
-                        print "debug: file '%s' exists indicating '%s' is running" % (globbed, self.name)
+                        logger.debug(
+                            "file '%s' exists indicating '%s' is running", globbed, self.name)
                         return True
             else:
                 raise RuntimeError(
@@ -183,185 +186,6 @@ class Cleaner:
         self.warnings[option_id] = description
 
 
-class Firefox(Cleaner):
-
-    """Mozilla Firefox"""
-
-    def __init__(self):
-        Cleaner.__init__(self)
-        self.add_option('backup', _('Backup files'), _(
-            'Delete the backup files'))
-        self.add_option('cache', _('Cache'), _(
-            'Delete the web cache, which reduces time to display revisited pages'))
-        self.add_option('cookies', _('Cookies'), _(
-            'Delete cookies, which contain information such as web site preferences, authentication, and tracking identification'))
-        self.add_option(
-            'crash_reports', _('Crash reports'), _('Delete the files'))
-        # TRANSLATORS: DOM = Document Object Model.
-        self.add_option('dom', _('DOM Storage'), _('Delete HTML5 cookies'))
-        self.add_option('download_history', _(
-            'Download history'), _('List of files downloaded'))
-        self.add_option('forms', _('Form history'), _(
-            'A history of forms entered in web sites and in the Search bar'))
-        self.add_option('session_restore', _('Session restore'), _(
-            'Loads the initial session after the browser closes or crashes'))
-        self.add_option('site_preferences', _(
-            'Site preferences'), _('Settings for individual sites'))
-        self.add_option('passwords', _('Passwords'), _(
-            'A database of usernames and passwords as well as a list of sites that should not store passwords'))
-        self.set_warning(
-            'passwords', _('This option will delete your saved passwords.'))
-        self.add_option(
-            'url_history', _('URL history'), _('List of visited web pages'))
-        self.add_option('vacuum', _('Vacuum'), _(
-            'Clean database fragmentation to reduce space and improve speed without removing any data'))
-
-        if 'posix' == os.name:
-            self.profile_dir = "~/.mozilla/firefox*/*.default*/"
-            self.add_running('exe', 'firefox')
-            self.add_running('exe', 'firefox-bin')
-            self.add_running('pathname', self.profile_dir + 'lock')
-        elif 'nt' == os.name:
-            self.profile_dir = "$USERPROFILE\\Application Data\\Mozilla\\Firefox\\Profiles\\*.default*\\"
-            self.add_running('exe', 'firefox.exe')
-
-        self.description = _("Web browser")
-        self.id = 'firefox'
-        self.name = "Firefox"
-
-    def get_commands(self, option_id):
-        files = []
-        # backup files
-        if 'backup' == option_id:
-            bookmark_bu_dir = os.path.join(self.profile_dir, 'bookmarkbackups')
-            files += FileUtilities.expand_glob_join(bookmark_bu_dir, "*.json")
-            files += FileUtilities.expand_glob_join(
-                bookmark_bu_dir, "*.jsonlz4")
-        # browser cache
-        cache_base = None
-        if 'posix' == os.name:
-            cache_base = self.profile_dir
-        elif 'nt' == os.name:
-            cache_base = "$localappdata\\Mozilla\\Firefox\\Profiles\\*.default*"
-        if 'cache' == option_id:
-            dirs = FileUtilities.expand_glob_join(cache_base, "Cache*")
-            dirs += FileUtilities.expand_glob_join(cache_base, "OfflineCache")
-            if 'nt' == os.name:
-                dirs += FileUtilities.expand_glob_join(
-                    cache_base, "jumpListCache")  # Windows 8
-            if 'posix' == os.name:
-                # This path is whitelisted under the System - Cache cleaner,
-                # so it can be cleaned here.
-                dirs += [os.path.expanduser('~/.cache/mozilla')]
-            for dirname in dirs:
-                for filename in children_in_directory(dirname, False):
-                    yield Command.Delete(filename)
-            # Necko Predictive Network Actions
-            # https://wiki.mozilla.org/Privacy/Reviews/Necko
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "netpredictions.sqlite")
-        # cookies
-        if 'cookies' == option_id:
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "cookies.txt")
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "cookies.sqlite")
-        # crash reports
-        if 'posix' == os.name:
-            crashdir = os.path.expanduser("~/.mozilla/firefox/Crash Reports")
-        if 'nt' == os.name:
-            crashdir = expandvars(
-                "$USERPROFILE\\Application Data\\Mozilla\\Firefox\\Crash Reports")
-        if 'crash_reports' == option_id:
-            for filename in children_in_directory(crashdir, False):
-                files += [filename]
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "minidumps/*.dmp")
-        # DOM storage
-        if 'dom' == option_id:
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "webappsstore.sqlite")
-
-        # download history
-        if 'download_history' == option_id:
-            # Firefox version 1
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "downloads.rdf")
-            # Firefox version 3
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "downloads.sqlite")
-        # forms
-        if 'forms' == option_id:
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "formhistory.dat")
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "formhistory.sqlite")
-        # passwords
-        if 'passwords' == option_id:
-            # http://kb.mozillazine.org/Password_Manager
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "signons.txt")
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "signons[2-3].txt")
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "signons.sqlite")
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "logins.json")
-        # session restore
-        if 'session_restore' == option_id:
-            # Names include sessionstore.js, sessionstore.bak,
-            # sessionstore.bak-20140715214327, sessionstore-1.js
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "sessionstore*.js")
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "sessionstore.bak*")
-            ss_bu_dir = os.path.join(self.profile_dir, 'sessionstore-backups')
-            files += FileUtilities.expand_glob_join(
-                ss_bu_dir, 'previous.js')
-            files += FileUtilities.expand_glob_join(
-                ss_bu_dir, 'upgrade.js-20*')
-            files += FileUtilities.expand_glob_join(
-                ss_bu_dir, 'recovery.js')
-            files += FileUtilities.expand_glob_join(
-                ss_bu_dir, 'recovery.bak')
-        # site-specific preferences
-        if 'site_preferences' == option_id:
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "content-prefs.sqlite")
-
-        # URL history
-        if 'url_history' == option_id:
-            # Firefox version 1
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "history.dat")
-            # Firefox 21 on Windows
-            if 'nt' == os.name:
-                files += FileUtilities.expand_glob_join(
-                    cache_base, "thumbnails/*.png")
-            # see also function other_cleanup()
-        # finish
-        for filename in files:
-            yield Command.Delete(filename)
-
-        # URL history
-        if 'url_history' == option_id:
-            for path in FileUtilities.expand_glob_join(self.profile_dir, "places.sqlite"):
-                yield Command.Function(path,
-                                       Special.delete_mozilla_url_history,
-                                       _('Delete the usage history'))
-
-        # vacuum
-        if 'vacuum' == option_id:
-            paths = []
-            paths += FileUtilities.expand_glob_join(
-                self.profile_dir, "*.sqlite")
-            if not cache_base == self.profile_dir:
-                paths += FileUtilities.expand_glob_join(cache_base, "*.sqlite")
-            for path in paths:
-                yield Command.Function(path,
-                                       FileUtilities.vacuum_sqlite3, _("Vacuum"))
-
-
 class OpenOfficeOrg(Cleaner):
 
     """Delete OpenOffice.org cache"""
@@ -526,28 +350,22 @@ class System(Cleaner):
         self.name = _("System")
 
     def get_commands(self, option_id):
-        # This variable will collect fully expanded file names, and
-        # at the end of this function, they will be checked they exist
-        # and processed through Command.Delete().
-        files = []
-
         # cache
         if 'posix' == os.name and 'cache' == option_id:
-            dirname = os.path.expanduser("~/.cache/")
+            dirname = expanduser("~/.cache/")
             for filename in children_in_directory(dirname, True):
-                if self.whitelisted(filename):
-                    continue
-                files += [filename]
+                if not self.whitelisted(filename):
+                    yield Command.Delete(filename)
 
         # custom
         if 'custom' == option_id:
             for (c_type, c_path) in options.get_custom_paths():
                 if 'file' == c_type:
-                    files += [c_path]
+                    yield Command.Delete(c_path)
                 elif 'folder' == c_type:
-                    files += [c_path]
+                    yield Command.Delete(c_path)
                     for path in children_in_directory(c_path, True):
-                        files += [path]
+                        yield Command.Delete(path)
                 else:
                     raise RuntimeError(
                         'custom folder has invalid type %s' % c_type)
@@ -622,7 +440,7 @@ class System(Cleaner):
             for path in paths:
                 expanded = expandvars(path)
                 for globbed in glob.iglob(expanded):
-                    files += [globbed]
+                    yield Command.Delete(globbed)
 
         # memory
         if sys.platform.startswith('linux') and 'memory' == option_id:
@@ -634,13 +452,15 @@ class System(Cleaner):
         if 'nt' == os.name and 'memory_dump' == option_id:
             fname = expandvars('$windir\\memory.dmp')
             if os.path.exists(fname):
-                files += [fname]
+                yield Command.Delete(fname)
             for fname in glob.iglob(expandvars('$windir\\Minidump\\*.dmp')):
-                files += [fname]
+                yield Command.Delete(fname)
 
         # most recently used documents list
         if 'posix' == os.name and 'recent_documents' == option_id:
-            files += [os.path.expanduser("~/.recently-used")]
+            ru_fn = expanduser("~/.recently-used")
+            if os.path.lexists(ru_fn):
+                yield Command.Delete(ru_fn)
             # GNOME 2.26 (as seen on Ubuntu 9.04) will retain the list
             # in memory if it is simply deleted, so it must be shredded
             # (or at least truncated).
@@ -650,12 +470,19 @@ class System(Cleaner):
             # shredding.
             #
             # https://bugzilla.gnome.org/show_bug.cgi?id=591404
+
+            def gtk_purge_items():
+                """Purge GTK items"""
+                gtk.RecentManager().purge_items()
+                yield 0
+
             for pathname in ["~/.recently-used.xbel", "~/.local/share/recently-used.xbel"]:
-                pathname = os.path.expanduser(pathname)
+                pathname = expanduser(pathname)
                 if os.path.lexists(pathname):
                     yield Command.Shred(pathname)
-                    if HAVE_GTK:
-                        gtk.RecentManager().purge_items()
+            if HAVE_GTK:
+                # Use the Function to skip when in preview mode
+                yield Command.Function(None, gtk_purge_items, _('Recent documents list'))
 
         if 'posix' == os.name and 'rotated_logs' == option_id:
             for path in Unix.rotated_logs():
@@ -676,21 +503,29 @@ class System(Cleaner):
 
         # temporary files
         if 'nt' == os.name and 'tmp' == option_id:
-            dirname = expandvars(
+            dirname1 = expandvars(
                 "$USERPROFILE\\Local Settings\\Temp\\")
+            dirname2 = expandvars(r'%temp%')
+            dirname3 = expandvars("%windir%\\temp\\")
+            dirnames = []
+            if Windows.get_windows_version() >= 6.0:
+                # Windows Vista or later
+                dirnames.append(dirname2)
+            else:
+                # Windows XP
+                dirnames.append(dirname1)
+            dirnames.append(dirname3)
             # whitelist the folder %TEMP%\Low but not its contents
             # https://bugs.launchpad.net/bleachbit/+bug/1421726
-            low = os.path.join(dirname, 'low').lower()
-            for filename in children_in_directory(dirname, True):
-                if not low == filename.lower():
-                    yield Command.Delete(filename)
-            dirname = expandvars("$windir\\temp\\")
-            for filename in children_in_directory(dirname, True):
-                yield Command.Delete(filename)
+            for dirname in dirnames:
+                low = os.path.join(dirname, 'low').lower()
+                for filename in children_in_directory(dirname, True):
+                    if not low == filename.lower():
+                        yield Command.Delete(filename)
 
         # trash
         if 'posix' == os.name and 'trash' == option_id:
-            dirname = os.path.expanduser("~/.Trash")
+            dirname = expanduser("~/.Trash")
             for filename in children_in_directory(dirname, False):
                 yield Command.Delete(filename)
             # fixme http://www.ramendik.ru/docs/trashspec.html
@@ -698,13 +533,13 @@ class System(Cleaner):
             # ~/.local/share/Trash
             # * GNOME 2.22, Fedora 9
             # * KDE 4.1.3, Ubuntu 8.10
-            dirname = os.path.expanduser("~/.local/share/Trash/files")
+            dirname = expanduser("~/.local/share/Trash/files")
             for filename in children_in_directory(dirname, True):
                 yield Command.Delete(filename)
-            dirname = os.path.expanduser("~/.local/share/Trash/info")
+            dirname = expanduser("~/.local/share/Trash/info")
             for filename in children_in_directory(dirname, True):
                 yield Command.Delete(filename)
-            dirname = os.path.expanduser("~/.local/share/Trash/expunged")
+            dirname = expanduser("~/.local/share/Trash/expunged")
             # desrt@irc.gimpnet.org tells me that the trash
             # backend puts files in here temporary, but in some situations
             # the files are stuck.
@@ -753,7 +588,9 @@ class System(Cleaner):
         # recycle bin
         if 'nt' == os.name and 'recycle_bin' == option_id:
             # This method allows shredding
+            recycled_any = False
             for path in Windows.get_recycle_bin():
+                recycled_any = True
                 yield Command.Delete(path)
             # If there were any files deleted, Windows XP will show the
             # wrong icon for the recycle bin indicating it is not empty.
@@ -765,27 +602,27 @@ class System(Cleaner):
             # opens the recycle bin folder.
 
             # This is a hack to refresh the icon.
-            import tempfile
-            tmpdir = tempfile.mkdtemp()
-            Windows.move_to_recycle_bin(tmpdir)
-            try:
-                Windows.empty_recycle_bin(None, True)
-            except:
-                logger = logging.getLogger(__name__)
-                logger.info('error in empty_recycle_bin()', exc_info=True)
+            def empty_recycle_bin_func():
+                import tempfile
+                tmpdir = tempfile.mkdtemp()
+                Windows.move_to_recycle_bin(tmpdir)
+                try:
+                    Windows.empty_recycle_bin(None, True)
+                except:
+                    logging.getLogger(__name__).info('error in empty_recycle_bin()', exc_info=True)
+                yield 0
+            # Using the Function Command prevents emptying the recycle bin
+            # when in preview mode.
+            if recycled_any:
+                yield Command.Function(None, empty_recycle_bin_func, _('Empty the recycle bin'))
 
         # Windows Updates
         if 'nt' == os.name and 'updates' == option_id:
             for wu in Windows.delete_updates():
                 yield wu
 
-        # return queued files
-        for filename in files:
-            if os.path.lexists(filename):
-                yield Command.Delete(filename)
-
-    def whitelisted(self, pathname):
-        """Return boolean whether file is whitelisted"""
+    def init_whitelist(self):
+        """Initialize the whitelist only once for performance"""
         regexes = [
             '^/tmp/.X0-lock$',
             '^/tmp/.truecrypt_aux_mnt.*/(control|volume)$',
@@ -799,19 +636,27 @@ class System(Cleaner):
             '^/tmp/orbit-[^/]+/bonobo-activation-register[a-z0-9-]*.lock$',
             '^/tmp/orbit-[^/]+/bonobo-activation-server-[a-z0-9-]*ior$',
             '^/tmp/pulse-[^/]+/pid$',
-            '^/var/tmp/kdecache-']
-        regexes.append('^' + os.path.expanduser('~/.cache/wallpaper/'))
-        # Clean Firefox cache from Firefox cleaner (LP#1295826)
-        regexes.append('^' + os.path.expanduser('~/.cache/mozilla'))
-        # Clean Google Chrome cache from Google Chrome cleaner (LP#656104)
-        regexes.append('^' + os.path.expanduser('~/.cache/google-chrome'))
-        regexes.append(
-            '^' + os.path.expanduser('~/.cache/gnome-control-center/'))
-        # iBus Pinyin
-        # https://bugs.launchpad.net/bleachbit/+bug/1538919
-        regexes.append('^' + os.path.expanduser('~/.cache/ibus/'))
+            '^/var/tmp/kdecache-',
+            '^' + expanduser('~/.cache/wallpaper/'),
+            # Clean Firefox cache from Firefox cleaner (LP#1295826)
+            '^' + expanduser('~/.cache/mozilla'),
+            # Clean Google Chrome cache from Google Chrome cleaner (LP#656104)
+            '^' + expanduser('~/.cache/google-chrome'),
+            '^' + expanduser('~/.cache/gnome-control-center/'),
+            # iBus Pinyin
+            # https://bugs.launchpad.net/bleachbit/+bug/1538919
+            '^' + expanduser('~/.cache/ibus/'),
+            # Linux Bluetooth daemon obexd
+            '^' + expanduser('~/.cache/obexd/')]
         for regex in regexes:
-            if None != re.match(regex, pathname):
+            self.regexes_compiled.append(re.compile(regex))
+
+    def whitelisted(self, pathname):
+        """Return boolean whether file is whitelisted"""
+        if not self.regexes_compiled:
+            self.init_whitelist()
+        for regex in self.regexes_compiled:
+            if regex.match(pathname) is not None:
                 return True
         return False
 
@@ -825,17 +670,16 @@ def register_cleaners():
     backends.clear()
 
     # initialize "hard coded" (non-CleanerML) backends
-    backends["firefox"] = Firefox()
     backends["openofficeorg"] = OpenOfficeOrg()
     backends["system"] = System()
 
     # register CleanerML cleaners
-    import CleanerML
+    from bleachbit import CleanerML
     CleanerML.load_cleaners()
 
     # register Winapp2.ini cleaners
     if 'nt' == os.name:
-        import Winapp
+        from bleachbit import Winapp
         Winapp.load_cleaners()
 
 
@@ -845,7 +689,7 @@ def create_simple_cleaner(paths):
     cleaner.add_option(option_id='files', name='', description='')
     cleaner.name = _("System")  # shows up in progress bar
 
-    import Action
+    from bleachbit import Action
 
     class CustomFileAction(Action.ActionProvider):
         action_key = '__customfileaction'
@@ -883,7 +727,7 @@ def create_wipe_cleaner(path):
             yield ret
         yield 0
 
-    import Action
+    from bleachbit import Action
 
     class CustomWipeAction(Action.ActionProvider):
         action_key = '__customwipeaction'
