@@ -3,8 +3,8 @@
 
 
 # BleachBit
-# Copyright (C) 2008-2016 Andrew Ziem
-# http://www.bleachbit.org
+# Copyright (C) 2008-2019 Andrew Ziem
+# https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,21 +24,24 @@
 Preferences dialog
 """
 
+from __future__ import absolute_import, print_function
+
+from bleachbit import _, _p, online_update_notification_enabled
+from bleachbit.Options import options
+from bleachbit import GuiBasic
 
 import gtk
+import logging
 import os
 import sys
 import traceback
 
-from Common import _, _p, online_update_notification_enabled
-from Options import options
-import GuiBasic
-
 if 'nt' == os.name:
-    import Windows
+    from bleachbit import Windows
 if 'posix' == os.name:
-    import Unix
+    from bleachbit import Unix
 
+logger = logging.getLogger(__name__)
 
 LOCATIONS_WHITELIST = 1
 LOCATIONS_CUSTOM = 2
@@ -153,7 +156,7 @@ class PreferencesDialog:
         # TRANSLATORS: Overwriting is the same as shredding.  It is a way
         # to prevent recovery of the data. You could also translate
         # 'Shred files to prevent recovery.'
-        cb_shred = gtk.CheckButton(_("Overwrite files to hide contents"))
+        cb_shred = gtk.CheckButton(_("Overwrite contents of files to prevent recovery"))
         cb_shred.set_active(options.get('shred'))
         cb_shred.connect('toggled', self.__toggle_callback, 'shred')
         cb_shred.set_tooltip_text(
@@ -312,7 +315,7 @@ class PreferencesDialog:
             if pathname:
                 for this_pathname in pathnames:
                     if pathname == this_pathname[1]:
-                        print "warning: '%s' already exists in whitelist" % pathname
+                        logger.warning("'%s' already exists in whitelist", pathname)
                         return
                 liststore.append([_('File'), pathname])
                 pathnames.append(['file', pathname])
@@ -326,7 +329,7 @@ class PreferencesDialog:
             if pathname:
                 for this_pathname in pathnames:
                     if pathname == this_pathname[1]:
-                        print "warning: '%s' already exists in whitelist" % pathname
+                        logger.warning("'%s' already exists in whitelist", pathname)
                         return
                 liststore.append([_('Folder'), pathname])
                 pathnames.append(['folder', pathname])
@@ -353,7 +356,7 @@ class PreferencesDialog:
             if pathname:
                 for this_pathname in pathnames:
                     if pathname == this_pathname[1]:
-                        print "warning: '%s' already exists in whitelist" % pathname
+                        logger.warning("'%s' already exists in whitelist", pathname)
                         return
                 liststore.append([_('File'), pathname])
                 pathnames.append(['file', pathname])
@@ -367,7 +370,7 @@ class PreferencesDialog:
             if pathname:
                 for this_pathname in pathnames:
                     if pathname == this_pathname[1]:
-                        print "warning: '%s' already exists in whitelist" % pathname
+                        logger.warning("'%s' already exists in whitelist", pathname)
                         return
                 liststore.append([_('Folder'), pathname])
                 pathnames.append(['folder', pathname])
